@@ -15,14 +15,31 @@ from wheezy.template.loader import FileLoader
 from collections import defaultdict
 from optparse import OptionParser
 
+import pkg_resources
 
-# Get current running script folder (MetaPath app folder)
-scriptdir = os.path.realpath(__file__).rpartition('/')[0]
+# Get current running script folder
+# scriptdir = os.path.realpath(__file__).rpartition('/')[0]
 
 
 # Standard definitions
 et.register_namespace('', 'http://pathvisio.org/GPML/2013a')
 
+GPML2SVG_XREF_URLS = {
+    'HMDB': 'http://www.hmdb.ca/metabolites/%s',
+    'WikiPathways': 'http://wikipathways.org/index.php/Pathway:%s',
+    'Entrez Gene': 'http://www.ncbi.nlm.nih.gov/gene/%s',
+    'PubChem-compound': 'http://pubchem.ncbi.nlm.nih.gov/summary/summary.cgi?cid=%s',
+    'Uniprot/TrEMBL': 'http://www.uniprot.org/uniprot/%s',
+    'KEGG Orthology': 'http://www.genome.jp/dbget-bin/www_bget?ko+%s',
+    'Kegg Compound': 'http://www.kegg.jp/entry/%s',
+    'MetaCyc Compound': 'http://biocyc.org/META/NEW-IMAGE?type=COMPOUND&object=%s',
+}
+
+
+class PkgLoader(object):
+    def load(self,name):
+        return pkg_resources.resource_string( __name__, 'gpml2svg.svg' ) 
+
 
 def get_edge( x, y, xo, yo, xt, yt):
     # Define a box by xo->xo+width, yo->yo+width
@@ -78,17 +95,10 @@ def ci_dict_get( d, k):
 def gpml2svg(
     gpml, 
     node_colors = {},
-    xrefs = {
-        'HMDB': 'http://www.hmdb.ca/metabolites/%s',
-        'WikiPathways': 'http://wikipathways.org/index.php/Pathway:%s',
-        'Entrez Gene': 'http://www.ncbi.nlm.nih.gov/gene/%s',
-        'PubChem-compound': 'http://pubchem.ncbi.nlm.nih.gov/summary/summary.cgi?cid=%s',
-        'Uniprot/TrEMBL': 'http://www.uniprot.org/uniprot/%s',
-        'KEGG Orthology': 'http://www.genome.jp/dbget-bin/www_bget?ko+%s',
-        'Kegg Compound': 'http://www.kegg.jp/entry/%s',
-    },
-    xrefs_synonyms = {}, # A conversion dictionary to generate additional XRefs
-    xrefs_classes = ['HMDB','Entrez Gene'], # A list of XRefs to use to generate data node classes (post-coloring/transformations)
+    xref_urls = GPML2SVG_XREF_URLS,
+    xref_synonyms = {}, # A conversion dictionary to generate additional XRefs
+                     # formatted as a dict of tuples  {('KEGG Compound','C00221'), ('MetaCyc','GLUCOSE')}
+    xref_synonyms_fn = None # A callback to return a tuple of target xref
     ):
 
 
@@ -99,10 +109,13 @@ def gpml2svg(
     gpml = re.sub(' xmlns="[^"]+"', '', gpml, count=1) # Strip namespace
 
     tree = et.fromstring(gpml.encode('utf-8'))
+    
     templateEngine = Engine(
-                loader=FileLoader( '.' ),
+                loader=PkgLoader(),
                 extensions=[CoreExtension()]
             )
+    template = templateEngine.get_template( 'gpml2svg.svg' )
+
 
     # Get root page canvas size, pathway name and copyright data from the header
     text = {}
@@ -139,7 +152,7 @@ def gpml2svg(
         }
         
     restyle_t = {
-        'Color': 'fill:%s',
+        'Color': 'fill:%s;',
     }
 
     for dn in data_nodes:
@@ -153,21 +166,34 @@ def gpml2svg(
         node_styles = get_styles( g, restyle_n )
         text_styles = get_styles( g, restyle_t )
 
-        # Data visualisation
-        xrs = list( dn.iterfind('Xref') )    
+        xrefs = dict()
+        xrs = list( dn.iterfind('Xref') )
         for xr in xrs:
-            if xr.attrib['ID'] in node_colors.keys():
-                node_styles.append( 'fill:%s; stroke:%s;' % (node_colors[ xr.attrib['ID']][0] , node_colors[ xr.attrib['ID'] ][0] ) )
-                text_styles.append( 'fill:%s;' % node_colors[ xr.attrib['ID']][1] )
-                break
-                
-        url = ''
-        xrs = list( dn.iterfind('Xref') )    
-        for xr in xrs:
-            if xr.attrib['Database'] in xrefs.keys():
-                url = xrefs[ xr.attrib['Database'] ] % xr.attrib['ID']             
+            xdb = xr.attrib['Database']
+            xid = xr.attrib['ID']
+            xrefs[ xdb ] = xid
+            # Process Xref synonyms
+            if ( xdb, xid ) in xref_synonyms.keys():
+                xref_extra = xref_synonyms[ ( xdb, xid ) ]
+                xrefs[ xref_extra[0] ] = xref_extra[1]
+        
+        if xref_synonyms_fn:
+            xrefs = xref_synonyms_fn( xrefs )
+            
+        url = None
+        for xdb, xid in xrefs.items():
+            if xdb in xref_urls.keys():
+                url = xref_urls[ xdb ] % xid     
                 break
 
+        # Data visualisation
+        # xrs = list( dn.iterfind('Xref') )    
+        for xdb,xid in xrefs.items():
+            if (xdb,xid) in node_colors.keys():
+                node_styles.append( 'fill:%s; stroke:%s;' % (node_colors[ (xdb,xid) ][0], node_colors[ (xdb,xid) ][0] ) )
+                text_styles.append( 'fill:%s;' % node_colors[ (xdb,xid) ][1] )
+                break
+                
         width  = float(g.attrib['Width'])
         height = float(g.attrib['Height'])
 
@@ -561,7 +587,6 @@ def gpml2svg(
         'graphids': graphids,
     }
 
-    template = templateEngine.get_template( os.path.join( scriptdir,'gpml2svg.svg') )
     return template.render( render_data ), metadata # return additional metadata
 
 def main():
